/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "buck7.h"

// ------------------------------------------------------------- PRIVATE MACROS 

#define BUCK7_DUMMY 0

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void buck7_cfg_setup ( buck7_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->sck = HAL_PIN_NC;
    cfg->miso = HAL_PIN_NC;
    cfg->mosi = HAL_PIN_NC;
    cfg->cs = HAL_PIN_NC;

    // Additional gpio pins

    cfg->en = HAL_PIN_NC;
    cfg->rst = HAL_PIN_NC;
    cfg->mod = HAL_PIN_NC;

    cfg->spi_speed = SPI_MEDIUM; 
    cfg->spi_mode = SPI_MODE_0;
}

BUCK7_RETVAL buck7_init ( buck7_t *ctx, buck7_cfg_t *cfg )
{
    spi_master_config_t spi_cfg;

    spi_master_configure_default( &spi_cfg );
    spi_cfg.mode        = cfg->spi_mode;
    spi_cfg.speed       = cfg->spi_speed;
    spi_cfg.pins.sck    = cfg->sck;
    spi_cfg.pins.miso   = cfg->miso;
    spi_cfg.pins.mosi   = cfg->mosi;
    spi_cfg.chip_select = cfg->cs;

    if (  spi_master_open( &ctx->spi, &spi_cfg ) != SPI_SUCCESS )
    {
        return BUCK7_INIT_ERROR;
    }

    spi_master_set_dummy_data( &ctx->spi, BUCK7_DUMMY );

    // Output pins 
    
    digital_out_init( &ctx->en, cfg->en );
    digital_out_init( &ctx->rst, cfg->rst );
    digital_out_init( &ctx->mod, cfg->mod );

    return BUCK7_OK;

}

void buck7_generic_transfer ( buck7_t *ctx, spi_master_transfer_data_t *block )
{
    spi_master_start( &ctx->spi );
    spi_master_transfer( &ctx->spi, block );
    spi_master_stop( &ctx->spi );    
}

void buck7_set_output_voltage ( buck7_t *ctx, uint16_t voltage )
{
    uint8_t write_volt[ 2 ];
    uint16_t set_volt;
    
    set_volt = 0x7000 | voltage;
    
    write_volt[ 0 ] = set_volt >> 8;
    write_volt[ 1 ] = set_volt & 0x00FF;
    
    spi_master_start( &ctx->spi );
    spi_master_write( &ctx->spi, write_volt[ 0 ] );
    spi_master_write( &ctx->spi, write_volt[ 1 ] );
    spi_master_stop( &ctx->spi );
}

void buck7_enable ( buck7_t *ctx )
{
    digital_out_high( &ctx->en );
}

void buck7_set_mode ( buck7_t *ctx, uint8_t mode )
{
    digital_out_write( &ctx->mod, mode );
}

uint8_t buck7_get_res_state ( buck7_t *ctx )
{
    return digital_in_read( &ctx->rst );
}

// ------------------------------------------------------------------------- END

